@@ -15,6 +15,9 @@
 #               2019/11/10 - PH Also write pitch to CameraElevationAngle
 #               2020/12/01 - PH Added ability to read DJI CSV log files
 #               2022/06/21 - PH Added ability to read Google Takeout JSON files
+#               2024/04/23 - PH Added ability to read more OpenTracks GPS tags
+#               2024/08/28 - PH Added support for new Google Takeout JSON format
+#               2024/11/26 - PH Also write GPSMeasureMode and GPSDOP
 #
 # References:   1) http://www.topografix.com/GPX/1/1/
 #               2) http://www.gpsinformation.org/dale/nmea.htm#GSA
@@ -29,7 +32,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:Public);
 use Image::ExifTool::GPS;
 
-$VERSION = '1.74';
+$VERSION = '1.81';
 
 sub JITTER() { return 2 }       # maximum time jitter
 
@@ -66,6 +69,8 @@ my %xmlTag = (
     course      => 'dir',       # (written by Arduino)
     pitch       => 'pitch',     # (written by Arduino)
     roll        => 'roll',      # (written by Arduino)
+    speed       => 'speed',     # (OpenTrack gpx)
+    accuracy_horizontal => 'err',#(OpenTrack gpx)
     # XML containers (fix is reset at the opening tag of these properties)
     wpt         => '',          # GPX
     trkpt       => '',          # GPX
@@ -85,12 +90,26 @@ my %fixInfoKeys = (
     alt    => [ 'alt' ],
     orient => [ 'dir', 'pitch', 'roll' ],
     atemp  => [ 'atemp' ],
+    err    => [ 'err' ],
+    dop    => [ 'hdop', 'vdop', 'pdop' ],
 );
 
-my %isOrient = ( dir => 1, pitch => 1, roll => 1 ); # test for orientation key
+# category for select keys
+my %keyCategory = (
+    dir => 'orient',
+    pitch => 'orient',
+    roll => 'orient',
+    hdop => 'dop',
+    pdop => 'dop',
+    vdop => 'dop',
+);
 
 # tags which may exist separately in some formats (eg. CSV)
-my %sepTags = ( dir => 1, pitch => 1, roll => 1, track => 1, speed => 1 );
+my %sepTags = (
+    dir => 1, pitch => 1, roll => 1, track => 1, speed => 1,
+    # (plus other tags we don't want to scan outwards for)
+    hdop => 1, pdop => 1, vdop => 1,
+);
 
 # conversion factors for GPSSpeed (standard EXIF units only)
 my %speedConv = (
@@ -147,7 +166,7 @@ sub LoadTrackLog($$;$)
     my ($raf, $from, $time, $isDate, $noDate, $noDateChanged, $lastDate, $dateFlarm);
     my ($nmeaStart, $fixSecs, @fixTimes, $lastFix, %nmea, @csvHeadings, $sortFixes);
     my ($canCut, $cutPDOP, $cutHDOP, $cutSats, $e0, $e1, @tmp, $trackFile, $trackTime);
-    my $scaleSpeed;
+    my ($scaleSpeed, $startTime);
 
     unless (eval { require Time::Local }) {
         return 'Geotag feature requires Time::Local installed';
@@ -314,10 +333,13 @@ sub LoadTrackLog($$;$)
                     }
                 }
                 next;
-            } elsif (/"(timelineObjects|placeVisit|activitySegment|latitudeE7)":/) {
+            } elsif (/"(timelineObjects|placeVisit|activitySegment|latitudeE7)"\s*:/) {
                 # Google Takeout JSON format
                 $format = 'JSON';
                 $sortFixes = 1; # (fixes are not all in order for this format)
+            } elsif (/"(durationMinutesOffsetFromStartTime|startTime)"\s*:/) {
+                $format = 'JSON';   # new Google Takeout JSON format (fixes seem to be in order)
+                $raf->Seek(0,0);    # rewind to start of file
             } else {
                 # search only first 50 lines of file for a valid fix
                 last if ++$skipped > 50;
@@ -340,14 +362,14 @@ sub LoadTrackLog($$;$)
                     my $tag = $xmlTag{lc $2};
                     if ($tag) {
                         $$fix{$tag} = $4;
-                        if ($isOrient{$tag}) {
-                            $$has{orient} = 1;
+                        if ($keyCategory{$tag}) {
+                            $$has{$keyCategory{$tag}} = 1;
                         } elsif ($tag eq 'alt') {
                             # validate altitude
                             undef $$fix{alt} if defined $$fix{alt} and $$fix{alt} !~ /^[+-]?\d+\.?\d*/;
                             $$has{alt} = 1 if $$fix{alt};   # set "has altitude" flag if appropriate
-                        } elsif ($tag eq 'atemp') {
-                            $$has{atemp} = 1;
+                        } elsif ($tag eq 'atemp' or $tag eq 'speed' or $tag eq 'err') {
+                            $$has{$tag} = 1;
                         }
                     }
                 }
@@ -386,14 +408,14 @@ sub LoadTrackLog($$;$)
                                 } else {
                                     $$fix{$tag} = $1;
                                 }
-                                if ($isOrient{$tag}) {
-                                    $$has{orient} = 1;
+                                if ($keyCategory{$tag}) {
+                                    $$has{$keyCategory{$tag}} = 1;
                                 } elsif ($tag eq 'alt') {
                                     # validate altitude
                                     undef $$fix{alt} if defined $$fix{alt} and $$fix{alt} !~ /^[+-]?\d+\.?\d*/;
                                     $$has{alt} = 1 if $$fix{alt};   # set "has altitude" flag if appropriate
-                                } elsif ($tag eq 'atemp') {
-                                    $$has{atemp} = 1;
+                                } elsif ($tag eq 'atemp' or $tag eq 'speed' or $tag eq 'err') {
+                                    $$has{$tag} = 1;
                                 }
                             }
                         }
@@ -554,14 +576,24 @@ DoneFix:    $isDate = 1;
             next;
         } elsif ($format eq 'JSON') {
             # Google Takeout JSON format
-            if (/"(latitudeE7|longitudeE7|latE7|lngE7|timestamp)":\s*"?(.*?)"?,?\s*[\x0d\x0a]/) {
-                if ($1 eq 'timestamp') {
+            if (/"(latitudeE7|longitudeE7|latE7|lngE7|timestamp|startTime|point|durationMinutesOffsetFromStartTime|time)"\s*:\s*"?(.*?)"?,?\s*[\x0d\x0a]/) {
+                if ($1 eq 'timestamp' or $1 eq 'time') {
                     $time = GetTime($2);
                     goto DoneFix if $time and $$fix{lat} and $$fix{lon};
+                } elsif ($1 eq 'startTime') { # (new format)
+                    $startTime = GetTime($2);
                 } elsif ($1 eq 'latitudeE7' or $1 eq 'latE7') {
                     $$fix{lat} = $2 * 1e-7;
-                } else {
+                } elsif ($1 eq 'longitudeE7' or $1 eq 'lngE7') {
                     $$fix{lon} = $2 * 1e-7;
+                } elsif ($1 eq 'point') { # (new format)
+                    my $point = $2;
+                    my @coords = $point =~ /[-+]?\d+\.\d+/g;
+                    @$fix{'lat','lon'} = @coords[0,1] if @coords == 2;
+                } elsif ($1 eq 'durationMinutesOffsetFromStartTime' and defined $startTime) { # (new format)
+                    $time = $startTime + $2 * 60;
+                    # note: this assumes that "point" comes first, which it does in my sample
+                    goto DoneFix if $time and $$fix{lat} and $$fix{lon};
                 }
             }
             next;
@@ -1109,8 +1141,9 @@ sub SetGeoValues($$;$)
                     $iExt = $i1;
                 }
                 if (abs($time - $tn) > $geoMaxExtSecs) {
-                    $err or $err = 'Time is too far from nearest GPS fix'.' '.abs($time-$tn).' '.$geoMaxExtSecs;
-                    $et->VPrint(2, '  Nearest fix:     ', PrintFixTime($tn), "\n") if $verbose > 2;
+                    $err or $err = 'Time is too far from nearest GPS fix';
+                    $et->VPrint(2, '  Nearest fix:     ', PrintFixTime($tn), ' (',
+                                int(abs $time-$tn), " sec away)\n") if $verbose > 2;
                     $fix = { } if $$geotag{DateTimeOnly};
                 } else {
                     $fix = $$points{$tn};
@@ -1126,7 +1159,7 @@ sub SetGeoValues($$;$)
                 # loop through available fix information categories
                 # (pos, track, alt, orient)
                 my ($category, $key);
-Category:       foreach $category (qw{pos track alt orient atemp}) {
+Category:       foreach $category (qw{pos track alt orient atemp err dop}) {
                     next unless $$has{$category};
                     my ($f, $p0b, $p1b, $f0b);
                     # loop through specific fix information keys
@@ -1218,16 +1251,29 @@ Category:       foreach $category (qw{pos track alt orient atemp}) {
             $coords .= " $alt";
         }
         @r = $et->SetNewValue(GPSCoordinates => $coords, %opts);
+        # also Geolocate if specified
+        my $nvHash;
+        my $geoloc = $et->GetNewValue('Geolocate', \$nvHash);
+        if ($geoloc and $geoloc =~ /\bgeotag\b/i) {
+            my $tag = ($$nvHash{WantGroup} ? "$$nvHash{WantGroup}:" : '') . 'Geolocate';
+            # pass along any regular expressions to qualify geolocation search
+            my $parms = join ',', grep m(/), split /\s*,\s*/, $geoloc;
+            $parms and $parms = ",$parms,both"; 
+            $et->SetNewValue($tag => "$$fix{lat},$$fix{lon}$parms");
+            # (the Geolocate tag will be restored to its original value
+            # by RestoreNewValues before the next file in batch processing)
+        }
         return $err if $qt; # all done if writing to QuickTime only
         # (capture error messages by calling SetNewValue in list context)
         @r = $et->SetNewValue(GPSLatitude => $$fix{lat}, %opts);
         @r = $et->SetNewValue(GPSLongitude => $$fix{lon}, %opts);
         @r = $et->SetNewValue(GPSAltitude => $gpsAlt, %opts);
         @r = $et->SetNewValue(GPSAltitudeRef => $gpsAltRef, %opts);
-        if ($$has{track}) {
+        if ($$has{track} or $$has{speed}) {
+            my $type = $$has{track} ? 'track' : 'speed';
             my $tFix = $fix;
-            if (not defined $$fix{track} and defined $iExt) {
-                my $p = FindFix($et,'track',$times,$points,$iExt,$iDir,$geoMaxExtSecs);
+            if (not defined $$fix{$type} and defined $iExt) {
+                my $p = FindFix($et,$type,$times,$points,$iExt,$iDir,$geoMaxExtSecs);
                 $tFix = $p if $p;
             }
             @r = $et->SetNewValue(GPSTrack => $$tFix{track}, %opts);
@@ -1268,6 +1314,28 @@ Category:       foreach $category (qw{pos track alt orient atemp}) {
             }
             @r = $et->SetNewValue(AmbientTemperature => $$tFix{atemp}, %opts);
         }
+        if ($$has{err}) {
+            @r = $et->SetNewValue(GPSHPositioningError => $$fix{err}, %opts);
+        }
+        if ($$has{dop}) {
+            my ($dop, $mm);
+            if (defined $$fix{pdop}) {
+                $dop = $$fix{pdop};
+                $mm = 3;
+            } elsif (defined $$fix{hdop}) {
+                if (defined $$fix{vdop}) {
+                    $dop = sqrt($$fix{hdop} * $$fix{hdop} + $$fix{vdop} * $$fix{vdop});
+                    $mm = 3;
+                } else {
+                    $dop = $$fix{hdop};
+                    $mm = 2;
+                }
+            }
+            if (defined $dop) {
+                $et->SetNewValue(GPSMeasureMode => $mm, %opts);
+                $et->SetNewValue(GPSDOP => $dop, %opts);
+            }
+        }
         unless ($xmp) {
             my ($latRef, $lonRef);
             $latRef = ($$fix{lat} > 0 ? 'N' : 'S') if defined $$fix{lat};
@@ -1293,7 +1361,8 @@ Category:       foreach $category (qw{pos track alt orient atemp}) {
                     GPSAltitude GPSAltitudeRef GPSDateStamp GPSTimeStamp GPSDateTime
                     GPSTrack GPSTrackRef GPSSpeed GPSSpeedRef GPSImgDirection
                     GPSImgDirectionRef GPSPitch GPSRoll CameraElevationAngle
-                    AmbientTemperature GPSCoordinates))
+                    AmbientTemperature GPSHPositioningError GPSCoordinates
+                    GPSMeasureMode GPSDOP))
         {
             my @r = $et->SetNewValue($_, undef, %opts);
         }
