@@ -16,7 +16,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::QuickTime;
 
-$VERSION = '1.08';
+$VERSION = '1.11';
 
 sub ProcessGoPro($$$);
 sub ProcessString($$$);
@@ -72,6 +72,7 @@ my %addUnits = (
         let me know if you discover the meaning of any of these unknown tags. See
         L<https://github.com/gopro/gpmf-parser> for details about this format.
     },
+  # ABSC (GPMF) - seen: 0 (fmt f)
     ACCL => { #2 (gpmd)
         Name => 'Accelerometer',
         Notes => 'accelerator readings in m/s2',
@@ -81,7 +82,9 @@ my %addUnits = (
   # ANGY (GPMF-GEOC) - seen 179.9 (fmt d, Max)
   # ANGZ (GPMF-GEOC) - seen 0.152 (fmt d, Max)
     ALLD => 'AutoLowLightDuration', #1 (gpmd) (untested)
-  # APTO (GPMF) - seen: 'RAW', 'DYNM' (fmt c)
+  # APTO (GPMF) - seen: 'OFF', 'RAW', 'DYNM' (fmt c)
+  # ARUW (GPMF) - seen: 1.14285719394684 (fmt f)
+  # ARWA (GPMF) - seen: 1.14285719394684 (fmt f)
     ATTD => { #PH (Karma)
         Name => 'Attitude',
         # UNIT=s,rad,rad,rad,rad/s,rad/s,rad/s,
@@ -96,8 +99,10 @@ my %addUnits = (
         # SCAL=1000 1 1 1 1
         Binary => 1,
     },
+  # AUBT (GPMF) - seen: ''N' (type c)
     AUDO => 'AudioSetting', #PH (GPMF - seen: 'WIND', fmt c)
   # AUPT (GPMF) - seen: 'N','Y' (fmt c)
+  # BITR (GPMF) - seen: 'STANDARD' (fmt c)
     BPOS => { #PH (Karma)
         Name => 'Controller',
         Unknown => 1,
@@ -111,9 +116,16 @@ my %addUnits = (
   # CALH (GPMF-GEOC) - seen 3040 (fmt L, Max)
   # CALW (GPMF-GEOC) - seen 4056 (fmt L, Max)
     CASN => 'CameraSerialNumber', #PH (GPMF - seen: 'C3221324545448', fmt c)
+  # CDAT (GPMF) - seen: 1732152823 (fmt J)
+  # CDTM (GPMF) - seen: 0 (fmt L)
   # CINF (GPMF) - seen: 0x67376be7709bc8876a8baf3940908618, 0xe230988539b30cf5f016627ae8fc5395,
   #                     0x8bcbe424acc5b37d7d77001635198b3b (fmt B) (Camera INFormation?)
+  # CLDP (GPMF) - seen: 'Y' (fmt c)
+  # CLKC (GPMF) - seen: 0 (fmt L)
+  # CLKS (GPMF) - seen: 2 (fmt B)
   # CMOD (GPMF) - seen: 12,13,17 [12 360 video, 13 time-laps video, 17 JPEG] (fmt B)
+  # CPID (GPMF) - seen: '1194885996 3387225026 733916448 2433577768' (fmt L)
+  # CPIN (GPMF) - seen: 1
   # CRTX (GPMF-BACK/FRNT) - double[1]
   # CRTY (GPMF-BACK/FRNT) - double[1]
     CSEN => { #PH (Karma)
@@ -123,6 +135,7 @@ my %addUnits = (
         # SCAL=1000 1 1 1 1 1 1 1 1 1 1
         Binary => 1,
     },
+  # CTRL (GPMF) - seen: 'Pro' (fmt c)
     CYTS => { #PH (Karma)
         Name => 'CoyoteStatus',
         # UNIT=s,,,,,rad,rad,rad,,
@@ -141,6 +154,8 @@ my %addUnits = (
         # (Max) DVID='FRNT',DVNM='Front Lens',KLNS,CTRX,CTRY,MFOV,SFTR
         # (Max) DVID='HLMT',DVNM='Highlights'
     },
+  # DNSC (GPMF) - seen: 'HIGH' (fmt c)
+  # DUST (GPMF) - seen: 'NO_LIMIT' (fmt c)
   # DVID (GPMF) - DeviceID; seen: 1 (fmt L), HLMT (fmt F), GEOC (fmt F), 'BACK' (fmt F, Max)
     DVID => { Name => 'DeviceID', Unknown => 1 }, #2 (gpmd)
   # DVNM (GPMF) seen: 'Video Global Settings' (fmt c), 'Highlights' (fmt c), 'Geometry Calibrations' (Max)
@@ -150,6 +165,7 @@ my %addUnits = (
         Name => 'DigitalZoom',
         PrintConv => { N => 'No', Y => 'Yes' },
     },
+  # DZMX (GPMF) - seen: 1.39999997615814 (fmt f)
   # DZST (GPMF) - seen: 0 (fmt L) (something to do with digital zoom maybe?)
     EISA => { #PH (GPMF) - seen: 'Y','N','HS EIS','N/A' (fmt c) [N was for a time-lapse video]
         Name => 'ElectronicImageStabilization',
@@ -188,10 +204,20 @@ my %addUnits = (
     },
     GPS5 => { #2 (gpmd)
         Name => 'GPSInfo',
+        # UNIT=deg,deg,m,m/s,m/s
+        # TYPE=l
         # SCAL=10000000,10000000,1000,1000,100
         RawConv => '$val', # necessary to use scaled value instead of raw data as subdir data
         SubDirectory => { TagTable => 'Image::ExifTool::GoPro::GPS5' },
     },
+    GPS9 => { #PH (gpmd, Hero 13)
+        Name => 'GPSInfo9',
+        # UNIT=deg,deg,m,m/s,m/s,,s,,
+        # TYPE=lllllllSS
+        # SCAL=10000000 10000000 1000 1000 100 1 1000 100 1
+        RawConv => '$val', # necessary to use scaled value instead of raw data as subdir data
+        SubDirectory => { TagTable => 'Image::ExifTool::GoPro::GPS9' },
+    },
     GPSF => { #2 (gpmd)
         Name => 'GPSMeasureMode',
         PrintConv => {
@@ -216,7 +242,10 @@ my %addUnits = (
         Notes => 'gyroscope readings in rad/s',
         Binary => 1,
     },
+  # HCTL (GPMF) - seen: "Off" (fmt c)
+  # HDRV (GPMF) - seen: "N" (fmt c)
   # HFLG (APP6) - seen: 0
+  # HSGT (GPMF) - seen: 'OFF' (fmt c)
     ISOE => 'ISOSpeeds', #PH (gpmd)
     ISOG => { #2 (gpmd)
         Name => 'ImageSensorGain',
@@ -232,6 +261,7 @@ my %addUnits = (
     },
   # KLNS (GPMF-BACK/FRNT) - double[5] (fmt d, Max)
   # LINF (GPMF) - seen: LAJ7061916601668,C3341326002180,C33632245450981 (fmt c) (Lens INFormation?)
+  # LMOD (GPMF) - seen: 'NONE' (fmt F)
     LNED => { #PH (Karma)
         Name => 'LocalPositionNED',
         # UNIT=s,m,m,m,m/s,m/s,m/s
@@ -240,14 +270,20 @@ my %addUnits = (
         Binary => 1,
     },
     MAGN => 'Magnetometer', #1 (gpmd) (units of uT)
+  # MAPX (GPMF) - seen: 1 (fmt f)
+  # MAPY (GPMF) - seen: 1 (fmt f)
   # MFOV (GPMF-BACK/FRNT) - seen: 100 (fmt d, Max)
     MINF => { #PH (GPMF - seen: HERO6 Black, fmt c)
         Name => 'Model',
         Groups => { 2 => 'Camera' },
         Description => 'Camera Model Name',
     },
+  # MMOD (GPMF) - seen: 'STEREO' (fmt c)
   # MTYP (GPMF) - seen: 0,1,5,11 [1 for time-lapse video, 5 for 360 video, 11 for JPEG] (fmt B)
   # MUID (GPMF) - seen: 3882563431 2278071152 967805802 411471936 0 0 0 0 (fmt L)
+  # MXCF (GPMF) - seen: 'x1' (fmt c)
+  # MYCF (GPMF) - seen: 'y1' (fmt c)
+  # ORDP (GPMF) - seen: 'Y' (fmt c)
     OREN => { #PH (GPMF - seen: 'U', fmt c)
         Name => 'AutoRotation',
         PrintConv => {
@@ -258,11 +294,16 @@ my %addUnits = (
     },
     # (most of the "P" tags are ProTune settings - PH)
     PHDR => 'HDRSetting', #PH (APP6 - seen: 0)
+  # PIMD (GPMF) - seen: 'AUTO' (fmt c)
     PIMN => 'AutoISOMin', #PH (GPMF - seen: 100, fmt L)
     PIMX => 'AutoISOMax', #PH (GPMF - seen: 1600, fmt L)
+  # POLY (GPMF) - seen: '0 2.11120247840881 0.14325800538063 -1.030...' (fmt f)
   # PRAW (APP6) - seen: 0, 'N', 'Y' (fmt c)
+  # PRCN (GPMF) - seen: 65 zeros (fmt B)
     PRES => 'PhotoResolution', #PH (APP6 - seen: '12MP_W')
   # PRJT (APP6) - seen: 'GPRO','EACO' (fmt F, Hero8, Max)
+  # PRNA (GPMF) - seen 10 (fmt B)
+  # PRNU (GPMF) - seen 0 (fmt B)
     PRTN => { #PH (GPMF - seen: 'N', fmt c)
         Name => 'ProTune',
         PrintConv => {
@@ -275,6 +316,9 @@ my %addUnits = (
     PTSH => 'Sharpness', #PH (GPMF - seen: 'HIGH', fmt c)
     PTWB => 'WhiteBalance', #PH (GPMF - seen: 'AUTO', fmt c)
   # PVUL (APP6) - seen: 'F' (fmt c, Hero8, Max)
+  # PWPR (GPMF) - seen: 'PERFORMANCE' (fmt c)
+  # PYCF (GPMF) - seen: '[r0,r1,r2,r3,r4,r5,r6]' (fmt c)
+  # RAMP (GPMF) - seen: empty string (fmt c)
     RATE => 'Rate', #PH (GPMF - seen: '0_5SEC', fmt c; APP6 - seen: '4_1SEC')
     RMRK => { #2 (gpmd)
         Name => 'Comments',
@@ -284,6 +328,7 @@ my %addUnits = (
         Name => 'ScaleFactor',
         Unknown => 1,
     },
+  # SCAP (GPMF) - seen: 'N' (fmt c)
     SCPR => { #PH (Karma) [stream was empty]
         Name => 'ScaledPressure',
         # UNIT=s,Pa,Pa,degC
@@ -291,6 +336,7 @@ my %addUnits = (
         # SCAL=1000 0.00999999977648258 0.00999999977648258 100
         %addUnits,
     },
+  # SCTM (GPMF) - seen 0 (fmt L)
   # SFTR (GPMF-BACK/FRNT) - seen 0.999,1.00004 (fmt d, Max)
   # SHFX (GPMF-GEOC) - seen 22.92 (fmt d, Max)
   # SHFY (GPMF-GEOC) - seen 0.123 (fmt d, Max)
@@ -355,6 +401,10 @@ my %addUnits = (
   # TOCK => { Name => 'OutTime', Unknown => 1, ValueConv => '$val/1000' }, #1 (gpmd)
     TSMP => { Name => 'TotalSamples', Unknown => 1 }, #2 (gpmd)
     TYPE => { Name => 'StructureType', Unknown => 1 }, #2 (gpmd,GPMF - eg 'LLLllfFff', fmt c)
+    TZON => { # (GPMF) - seen: 60 (fmt s)
+        Name => 'TimeZone',
+        PrintConv => 'Image::ExifTool::TimeZoneString($val)',
+    },
     UNIT => { #2 (gpmd) alternative units
         Name => 'Units',
         Unknown => 1,
@@ -373,6 +423,7 @@ my %addUnits = (
         },
     },
   # VLTA (GPMF) - seen: 78 ('N') (fmt B -- wrong format?)
+    VFPS => { Name => 'VideoFrameRate', PrintConv => '$val=~s( )(/);$val' }, #PH (GPMF, fmt L)
     VFRH => { #PH (Karma)
         Name => 'VisualFlightRulesHUD',
         BinaryData => 1,
@@ -380,13 +431,17 @@ my %addUnits = (
         # TYPE=ffffsS
     },
   # VLTE (GPMF) - seen: 'Y','N' (fmt c)
+    VRES => { Name => 'VideoFrameSize', PrintConv => '$val=~s/ /x/;$val' }, #PH (GPMF, fmt L)
     WBAL => 'ColorTemperatures', #PH (gpmd)
     WRGB => { #PH (gpmd)
         Name => 'WhiteBalanceRGB',
         Binary => 1,
     },
   # ZFOV (APP6,GPMF) - seen: 148.34, 0 (fmt f, Hero8, Max)
-  # the following ref forum12825
+  # ZMPL (GPMF) - seen: 0.652929663658142 (fmt f)
+#
+# the following ref forum12825
+#
     MUID => {
         Name => 'MediaUniqueID',
         PrintConv => q{
@@ -418,7 +473,7 @@ my %addUnits = (
     YAVG => 'LumaAverage',
 );
 
-# GoPro GPS5 tags (ref 2) (Hero5,Hero6)
+# GoPro GPS5 tags (ref 2) (Hero5,Hero6,Hero9)
 %Image::ExifTool::GoPro::GPS5 = (
     PROCESS_PROC => \&ProcessString,
     GROUPS => { 1 => 'GoPro', 2 => 'Location' },
@@ -435,8 +490,65 @@ my %addUnits = (
         Name => 'GPSAltitude',
         PrintConv => '"$val m"',
     },
-    3 => 'GPSSpeed',   # (unit='m/s')
-    4 => 'GPSSpeed3D', # (unit='m/s')
+    3 => {
+        Name => 'GPSSpeed',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+    4 => {
+        Name => 'GPSSpeed3D',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+);
+
+# GoPro GPS9 tags (Hero13)
+%Image::ExifTool::GoPro::GPS9 = (
+    PROCESS_PROC => \&ProcessString,
+    GROUPS => { 1 => 'GoPro', 2 => 'Location' },
+    VARS => { HEX_ID => 0, ID_LABEL => 'Index' },
+    0 => { # (unit='deg')
+        Name => 'GPSLatitude',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "N")',
+    },
+    1 => { # (unit='deg')
+        Name => 'GPSLongitude',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "E")',
+    },
+    2 => { # (unit='m')
+        Name => 'GPSAltitude',
+        PrintConv => '"$val m"',
+    },
+    3 => {
+        Name => 'GPSSpeed',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+    4 => {
+        Name => 'GPSSpeed3D',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+    5 => { # days since 2000
+        Name => 'GPSDays',
+        RawConv => '$$self{GPSDays} = $val; undef',
+        Hidden => 1,
+    },
+    6 => { # seconds of date/time
+        Name => 'GPSDateTime',
+        Groups => { 2 => 'Time' },
+        # (10957 days from Jan 1 1970 to Jan 1 2000)
+        RawConv => 'ConvertUnixTime(($$self{GPSDays} + 10957) * 24 * 3600 + $val, undef, 3)',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    7 => 'GPSDOP',
+    8 => {
+        Name => 'GPSMeasureMode', #PH (NC)
+        PrintConv => {
+            2 => '2-Dimensional Measurement',
+            3 => '3-Dimensional Measurement',
+        },
+    },
 );
 
 # GoPro GPRI tags (ref PH) (Karma)
@@ -604,7 +716,7 @@ sub ScaleValues($$)
 sub AddUnits($$$)
 {
     my ($et, $val, $tag) = @_;
-    if ($et and $$et{TAG_EXTRA}{$tag} and $$et{TAG_EXTRA}{$tag}{Units}) {
+    if ($et and $$et{TAG_EXTRA}{$tag}{Units}) {
         my $u = $$et{TAG_EXTRA}{$tag}{Units};
         $u = [ $u ] unless ref $u eq 'ARRAY';
         my @a = split ' ', $val;
