@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.36';
+$VERSION = '1.43';
 
 sub ProcessJpeg2000Box($$$);
 sub ProcessJUMD($$$);
@@ -141,10 +141,13 @@ my %j2cMarker = (
         Authenticity Initiative) JUMBF (JPEG Universal Metadata Box Format) metdata
         is currently extracted from JPEG, PNG, TIFF-based (eg. TIFF, DNG),
         QuickTime-based (eg. MP4, MOV, HEIF, AVIF), RIFF-based (eg. WAV, AVI, WebP),
-        GIF files and ID3v2 metadata.  The suggested ExifTool command-line arguments
-        for reading C2PA metadata are C<-jumbf:all -G3 -b -j -u -struct>.  This
-        metadata may be deleted from writable JPEG, PNG, WebP, TIFF-based, and
-        QuickTime-based files by deleting the JUMBF group with C<-jumbf:all=>.
+        PDF, SVG and GIF files, and ID3v2 metadata.  The suggested ExifTool
+        command-line arguments for reading C2PA metadata are C<-jumbf:all -G3 -b -j
+        -u -struct>.  This metadata may be deleted from writable JPEG, PNG, WebP,
+        TIFF-based, and QuickTime-based files by deleting the JUMBF group with
+        C<-jumbf:all=>.  The C2PA JUMBF metadata may be extracted as a block via the
+        JUMBF tag.  See L<https://c2pa.org/specifications/> for the C2PA
+        specification.
     },
 #
 # NOTE: ONLY TAGS WITH "Format" DEFINED ARE EXTRACTED!
@@ -419,7 +422,7 @@ my %j2cMarker = (
         Flags => [ 'Binary', 'Protected' ],
         SubDirectory => { TagTable => 'Image::ExifTool::CBOR::Main' },
     },
-    bfdb => { # used in JUMBF (see  # (used when tag is renamed according to JUMDLabel)
+    bfdb => { # used in JUMBF
         Name => 'BinaryDataType',
         Notes => 'JUMBF, MIME type and optional file name',
         Format => 'undef',
@@ -571,6 +574,7 @@ my %j2cMarker = (
     2 => {
         Name => 'CompatibleBrands',
         Format => 'undef[$size-8]',
+        List => 1, # (for documentation only)
         # ignore any entry with a null, and return others as a list
         ValueConv => 'my @a=($val=~/.{4}/sg); @a=grep(!/\0/,@a); \@a',
     },
@@ -627,6 +631,7 @@ my %j2cMarker = (
 %Image::ExifTool::Jpeg2000::ColorSpec = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData, # (we don't actually call this)
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
     GROUPS => { 2 => 'Image' },
     FORMAT => 'int8s',
     WRITABLE => 1,
@@ -724,7 +729,6 @@ my %j2cMarker = (
         {
             Name => 'ColorSpecData',
             Format => 'undef[$size-3]',
-            Writable => 'undef',
             Protected => 1,
             Binary => 1,
         },
@@ -748,6 +752,7 @@ my %j2cMarker = (
         },
         # seen:
         # cacb/cast/caas/cacl/casg/json-00110010800000aa00389b71
+        #   (also brob- but not yet tested)
         # 6579d6fbdba2446bb2ac1b82feeb89d1 - JPEG image
     },
     'label' => { Name => 'JUMDLabel' },
@@ -821,6 +826,7 @@ sub ProcessJUMD($$$)
             $name =~ tr/-_a-zA-Z0-9//dc;    # remove other illegal characters
             $name =~ s/__/_/;               # collapse double underlines
             $name = ucfirst $name;          # capitalize first letter
+            $name =~ s/C2pa/C2PA/;          # capitalize C2PA
             $name = "Tag$name" if length($name) < 2; # must at least 2 characters long
             $$et{JUMBFLabel} = $name;
         }
@@ -854,6 +860,17 @@ sub ProcessJUMD($$$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Warn about error in Brotli compression/decompression
+# Inputs: 0) ExifTool ref, 1) box type, 2) true for decoding (Uncompress)
+sub BrotliWarn($$;$)
+{
+    my ($et, $type, $uncompress) = @_;
+    my ($enc, $mod) = $uncompress ? qw(decoding Uncompress) : qw(encoding Compress);
+    $et->Warn("Error $enc '${type}' brob box");
+    $et->Warn("Try updating to IO::${mod}::Brotli 0.004 or later");
+}
+
 #------------------------------------------------------------------------------
 # Create new JPEG 2000 boxes when writing
 # (Currently only supports adding top-level Writable JPEG2000 tags and certain UUID boxes)
@@ -909,7 +926,7 @@ sub CreateNewBoxes($$)
                         my $compressed;
                         eval { $compressed = IO::Compress::Brotli::bro($pad . $newdir) };
                         if ($@ or not $compressed) {
-                            $et->Warn("Error encoding $dirName brob box");
+                            BrotliWarn($et, $dirName);
                         } else {
                             $et->VPrint(0, "  Writing Brotli-compressed $dir\n");
                             $newdir = $compressed;
@@ -917,7 +934,7 @@ sub CreateNewBoxes($$)
                             $tag = 'brob';
                         }
                     } else {
-                        $et->WarnOnce('Install IO::Compress::Brotli to create Brotli-compressed metadata');
+                        $et->Warn('Install IO::Compress::Brotli to create Brotli-compressed metadata');
                     }
                 }
                 my $boxhdr = pack('N', length($newdir) + length($pad) + 8) . $tag;
@@ -1006,11 +1023,23 @@ sub ProcessJpeg2000Box($$$)
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $base = $$dirInfo{Base} || 0;
     my $outfile = $$dirInfo{OutFile};
+    my $dirName = $$dirInfo{DirName} || '';
     my $dirEnd = $dirStart + $dirLen;
     my ($err, $outBuff, $verbose, $doColour, $hash, $raf);
 
-    # read from RAF unless reading from buffer
-    $raf = $$dirInfo{RAF} unless $dataPt;
+    if ($dataPt) {
+        # save C2PA JUMBF as a block if requested
+        if ($dirName eq 'JUMBF' and $$et{REQ_TAG_LOOKUP}{jumbf} and not $$dirInfo{NoBlockSave}) {
+            if ($dirStart or $dirLen ne length($$dataPt)) {
+                my $dat = substr($$dataPt, $dirStart, $dirLen);
+                $et->FoundTag(JUMBF => \$dat);
+            } else {
+                $et->FoundTag(JUMBF => $dataPt);
+            }
+        }
+    } else {
+        $raf = $$dirInfo{RAF};  # read from RAF
+    }
 
     if ($outfile) {
         unless ($raf) {
@@ -1019,7 +1048,7 @@ sub ProcessJpeg2000Box($$$)
             $outfile = \$outBuff;
         }
         # determine if we will be writing colr box
-        if ($$dirInfo{DirName} and $$dirInfo{DirName} eq 'JP2Header') {
+        if ($dirName eq 'JP2Header') {
             $doColour = 2 if defined $et->GetNewValue('ColorSpecMethod') or $et->GetNewValue('ICC_Profile') or
                              defined $et->GetNewValue('ColorSpecPrecedence') or defined $et->GetNewValue('ColorSpace') or
                              defined $et->GetNewValue('ColorSpecApproximation') or defined $et->GetNewValue('ColorSpecData');
@@ -1027,7 +1056,7 @@ sub ProcessJpeg2000Box($$$)
     } else {
         # (must not set verbose flag when writing!)
         $verbose = $$et{OPTIONS}{Verbose};
-        $et->VerboseDir($$dirInfo{DirName}) if $verbose;
+        $et->VerboseDir($dirName) if $verbose;
         # do hash if requested, but only for top-level image data
         $hash = $$et{ImageDataHash} if $raf;
     }
@@ -1176,7 +1205,7 @@ sub ProcessJpeg2000Box($$$)
         # create new tag for JUMBF data values with name corresponding to JUMBFLabel
         if ($tagInfo and $$et{JUMBFLabel} and (not $$tagInfo{SubDirectory} or $$tagInfo{BlockExtract})) {
             $tagInfo = { %$tagInfo, Name => $$et{JUMBFLabel} . ($$tagInfo{JUMBF_Suffix} || '') };
-            delete $$tagInfo{Description};
+            ($$tagInfo{Description} = Image::ExifTool::MakeDescription($$tagInfo{Name})) =~ s/C2 PA/C2PA/;
             AddTagToTable($tagTablePtr, '_JUMBF_' . $$et{JUMBFLabel}, $tagInfo);
             delete $$tagInfo{Protected}; # (must do this so -j -b returns JUMBF binary data)
             $$tagInfo{TagID} = $boxID;
@@ -1248,7 +1277,7 @@ sub ProcessJpeg2000Box($$$)
                                 my $compressed;
                                 eval { $compressed = IO::Compress::Brotli::bro($pad . $newdir) };
                                 if ($@ or not $compressed) {
-                                    $et->Warn("Error encoding $boxID brob box");
+                                    BrotliWarn($et, $boxID);
                                 } else {
                                     $et->VPrint(0, "  Writing Brotli-compressed $boxID\n");
                                     $newdir = $boxID . $compressed;
@@ -1257,7 +1286,7 @@ sub ProcessJpeg2000Box($$$)
                                     ++$$et{CHANGED};
                                 }
                             } else {
-                                $et->WarnOnce('Install IO::Compress::Brotli to write Brotli-compressed metadata');
+                                $et->Warn('Install IO::Compress::Brotli to write Brotli-compressed metadata');
                             }
                         } elsif (not $compress and $boxID eq 'brob') {
                             # (in this case, ProcessBrotli has returned uncompressed data,
@@ -1310,7 +1339,7 @@ sub ProcessJpeg2000Box($$$)
             if (defined $val) {
                 my $key = $et->FoundTag($tagInfo, $val);
                 # save Rational value
-                $$et{RATIONAL}{$key} = $rational if defined $rational and defined $key;
+                $$et{TAG_EXTRA}{$key}{Rational} = $rational if defined $rational and defined $key;
             }
         } elsif ($outfile) {
             my $boxhdr = pack('N', $boxLen + 8) . $boxID;
@@ -1382,14 +1411,14 @@ sub ProcessBrotli($$$)
     }
     if (eval { require IO::Uncompress::Brotli }) {
         if ($isWriting and not eval { require IO::Compress::Brotli }) {
-            $et->WarnOnce('Install IO::Compress::Brotli to write Brotli-compressed metadata');
+            $et->Warn('Install IO::Compress::Brotli to write Brotli-compressed metadata');
             return undef;
         }
         my $compress = $et->Options('Compress');
         my $verbose = $isWriting ? 0 : $et->Options('Verbose');
         my $dat = substr($$dataPt, 4);
         eval { $dat = IO::Uncompress::Brotli::unbro($dat, 100000000) };
-        $@ and $et->Warn("Error decoding $type brob box"), return 1;
+        $@ and BrotliWarn($et, $type, 1), return 1;
         $verbose > 2 and $et->VerboseDump(\$dat, Prefix => $$et{INDENT} . '  ');
         my %dirInfo = ( DataPt => \$dat );
         if ($type eq 'xml ') {
@@ -1415,19 +1444,19 @@ sub ProcessBrotli($$$)
             }
         } elsif ($type eq 'jumb') {
             return undef if $isWriting; # (can't yet write JUMBF)
-            Image::ExifTool::ProcessJUMB($et, \%dirInfo, $tagTablePtr); # (untested)
+            Image::ExifTool::Jpeg2000::ProcessJUMB($et, \%dirInfo, $tagTablePtr); # (untested)
         }
         if ($isWriting) {
             return undef unless defined $dat;
             # rewrite as uncompressed if Compress option is set to 0 (or '')
             return $dat if defined $compress and not $compress;
             eval { $dat = IO::Compress::Brotli::bro($dat) };
-            $@ and $et->Warn("Error encoding $type brob box"), return undef;
+            $@ and BrotliWarn($et, $type), return undef;
             $et->VPrint(0, "  Writing Brotli-compressed $type\n");
             return $type . $dat;
         }
     } else {
-        $et->WarnOnce('Install IO::Uncompress::Brotli to decode Brotli-compressed metadata');
+        $et->Warn('Install IO::Uncompress::Brotli to decode Brotli-compressed metadata');
         return undef if $isWriting;
     }
     return 1;
@@ -1480,6 +1509,28 @@ sub ProcessJXLCodestream($$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Read/write meta information from a C2PA/JUMBF file
+# Inputs: 0) ExifTool object reference, 1) dirInfo reference
+# Returns: 1 on success, 0 if this wasn't a valid JUMBF file
+sub ProcessJUMBF($$)
+{
+    my ($et, $dirInfo) = @_;
+    my $raf = $$dirInfo{RAF};
+    my $hdr;
+
+    # check to be sure this is a valid JPG2000 file
+    return 0 unless $raf->Read($hdr,20) == 20 and $raf->Seek(0,0);
+    return 0 unless $hdr =~ /^.{4}jumb\0.{3}jumd(.{4})/;
+    $et->SetFileType($1 eq 'c2pa' ? 'C2PA' : 'JUMBF');
+    my %dirInfo = (
+        RAF => $raf,
+        DirName => 'JUMBF',
+    );
+    my $tagTablePtr = GetTagTable('Image::ExifTool::Jpeg2000::Main');
+    return $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
+}
+
 #------------------------------------------------------------------------------
 # Read/write meta information from a JPEG 2000 image
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference
@@ -1494,7 +1545,7 @@ sub ProcessJP2($$)
 
     # check to be sure this is a valid JPG2000 file
     return 0 unless $raf->Read($hdr,12) == 12;
-    unless ($hdr eq "\0\0\0\x0cjP  \x0d\x0a\x87\x0a" or     # (ref 1)
+    unless ($hdr eq "\0\0\0\x0cjP  \x0d\x0a\x87\x0a" or       # (ref 1)
             $hdr eq "\0\0\0\x0cjP\x1a\x1a\x0d\x0a\x87\x0a" or # (ref 2)
             $$et{IsJXL})
     {
@@ -1572,7 +1623,7 @@ sub ProcessJXL($$)
             $$et{IsJXL} = 2;
             my $buff = "\0\0\0\x0cJXL \x0d\x0a\x87\x0a\0\0\0\x14ftypjxl \0\0\0\0jxl ";
             # add metadata to empty ISO BMFF container
-            $$dirInfo{RAF} = new File::RandomAccess(\$buff);
+            $$dirInfo{RAF} = File::RandomAccess->new(\$buff);
         } else {
             $et->SetFileType('JXL Codestream','image/jxl', 'jxl');
             if ($$et{ImageDataHash} and $raf->Seek(0,0)) {
@@ -1620,7 +1671,7 @@ files.
 
 =head1 AUTHOR
 
-Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
